#include "Tele.hh"

Tele::Tele(Int_t runnum, TString RootPath, TString TelePath)
{
	TString RootFullName="";
	TString TeleFullName="";
	//root file
	char rootfilename[36];
	sprintf(rootfilename,"/data%04d.root",runnum);
	RootFullName=RootPath+rootfilename;

	pRootFile=new TFile(RootFullName.Data(),"read");
	if(!pRootFile->IsOpen())
	{
		cout<<"Cannot open input file with runnum is "<<runnum<<endl;
		exit(0);
	}
	pRootTree=(TTree*)pRootFile->Get("tree");
	if(!pRootTree)
	{
		cout<<"Cannot find tree named 'tree' ! "<<endl;
		exit(0);		
	}
	//tele file	
	char telefilename[36];
	sprintf(telefilename,"/tele%04d.root",runnum);
	TeleFullName=TelePath+telefilename;
	pTeleFile=new TFile(TeleFullName.Data(),"recreate");
	pTeleTree=new TTree("tree","tele");
	Init();
}

Tele::~Tele()
{

}

void Tele::Init()
{
	//root file
	pRootTree->SetBranchAddress("ahit", &ahit, &b_ahit);
	pRootTree->SetBranchAddress("amod", amod, &b_amod);
	pRootTree->SetBranchAddress("ach", ach, &b_ach);
	pRootTree->SetBranchAddress("aval", aval, &b_aval);
	pRootTree->SetBranchAddress("adc", adc, &b_adc);
	pRootTree->SetBranchAddress("acnt", acnt, &b_acnt);
	pRootTree->SetBranchAddress("mhit", &mhit, &b_mhit);
	pRootTree->SetBranchAddress("mmod", mmod, &b_mmod);
	pRootTree->SetBranchAddress("mch", mch, &b_mch);
	pRootTree->SetBranchAddress("mval", mval, &b_mval);
	pRootTree->SetBranchAddress("madc", madc, &b_madc);
	pRootTree->SetBranchAddress("mcnt", &mcnt, &b_mcnt);
	pRootTree->SetBranchAddress("ghit", &ghit, &b_ghit);
	pRootTree->SetBranchAddress("gmod", gmod, &b_gmod);
	pRootTree->SetBranchAddress("gch", gch, &b_gch);
	pRootTree->SetBranchAddress("gval", gval, &b_gval);
	pRootTree->SetBranchAddress("gdc", gdc, &b_gdc);
	pRootTree->SetBranchAddress("gmulti", gmulti, &b_gmulti);
	pRootTree->SetBranchAddress("gcnt", gcnt, &b_gcnt);
	pRootTree->SetBranchAddress("shit", &shit, &b_shit);
	pRootTree->SetBranchAddress("smod", smod, &b_smod);
	pRootTree->SetBranchAddress("sch", sch, &b_sch);
	pRootTree->SetBranchAddress("sval", sval, &b_sval);
	pRootTree->SetBranchAddress("sdc", sdc, &b_sdc);
	pRootTree->SetBranchAddress("nevt", &nevt, &b_nevt);	
	//tele file
	r2t1=new DSSD();
	r2t2=new DSSD();
	l2t1=new DSSD();
	l2t2=new DSSD();

	r1t1=new SSSD();
	r1t2=new SSSD();
	l1t1=new SSSD();
	l1t2=new SSSD();

	r3t1=new SSD();
	r3t2=new SSD();
	l3t1=new SSD();
	l3t2=new SSD();
	
	pTeleTree->Branch("r2t1",r2t1);
	pTeleTree->Branch("r2t2",r2t2);
	pTeleTree->Branch("l2t1",l2t1);
	pTeleTree->Branch("l2t2",l2t2);
	
	pTeleTree->Branch("r1t1",r1t1);
	pTeleTree->Branch("r1t2",r1t2);
	pTeleTree->Branch("l1t1",l1t1);
	pTeleTree->Branch("l1t2",l1t2);

	pTeleTree->Branch("r3t1",r3t1);
	pTeleTree->Branch("r3t2",r3t2);
	pTeleTree->Branch("l3t1",l3t1);
	pTeleTree->Branch("l3t2",l3t2);

	pTeleTree->Branch("tCoin",&tCoin,"tCoin/I");
	pTeleTree->Branch("tDTM2",&tDTM2,"tDTM2/I");
	pTeleTree->Branch("tT2db",&tT2db,"tT2db/I");
	pTeleTree->Branch("tT1db",&tT1db,"tT1db/I");


	tDTM2=-1;
	tCoin=-1;
	tT2db=-1;
	tT1db=-1;

}

void Tele::Loop(Int_t maxnevt)
{
	//For DSSD
	ReadPar("./cali/r2t2_xpar.txt",r2t2_xpar,32);
	ReadPar("./cali/r2t2_ypar.txt",r2t2_ypar,32);

	ReadPar("./cali/r2t1_xpar.txt",r2t1_xpar,32);
	ReadPar("./cali/r2t1_ypar.txt",r2t1_ypar,32);

	ReadPar("./cali/l2t1_xpar.txt",l2t1_xpar,32);
	ReadPar("./cali/l2t1_ypar.txt",l2t1_ypar,32);

	ReadPar("./cali/l2t2_xpar.txt",l2t2_xpar,32);
	ReadPar("./cali/l2t2_ypar.txt",l2t2_ypar,32);

	//For SSSD
	ReadPar("./cali/r1t2_xpar.txt",r1t2_xpar,16);
	ReadPar("./cali/r1t1_xpar.txt",r1t1_xpar,16);
	ReadPar("./cali/l1t1_xpar.txt",l1t1_xpar,16);
	ReadPar("./cali/l1t2_xpar.txt",l1t2_xpar,16);

	//For SSD
	ReadPar("./cali/r3t2_par.txt",r3t2_par,1);
	ReadPar("./cali/r3t1_par.txt",r3t1_par,1);
	ReadPar("./cali/l3t1_par.txt",l3t1_par,1);
	ReadPar("./cali/l3t2_par.txt",l3t2_par,1);


	Long64_t nentries = pRootTree->GetEntriesFast();
	maxnevt=maxnevt>0?maxnevt:nentries;
	for (Long64_t jentry=0; jentry<maxnevt;jentry++) 
	{
		Clear();
		Long64_t centry = pRootTree->LoadTree(jentry);
		if (centry < 0) break;
		pRootTree->GetEntry(jentry); 
		MappingDSSD();
		MappingSSSD();
		MappingSSD();
		tCoin=gmulti[0][112];
		tDTM2=gmulti[0][113];
		tT2db=gmulti[0][114];
		tT1db=gmulti[0][115];

		pTeleTree->Fill();
		// if (Cut(centry) < 0) continue;
		if(jentry%10000==0) std::cout<<"Processing.... "<<jentry<<"/"<<maxnevt<<endl;

	}
	pTeleTree->Write();
}

void Tele::Clear()
{
	r2t2->Clear();
	r2t1->Clear();
	l2t1->Clear();
	l2t2->Clear();

	r1t2->Clear();
	r1t1->Clear();
	l1t1->Clear();
	l1t2->Clear();
	
	r3t2->Clear();
	r3t1->Clear();
	l3t1->Clear();
	l3t2->Clear();
	tDTM2=-1;
	tCoin=-1;
	tT2db=-1;
	tT1db=-1;
}

void Tele::MappingDSSD()
{
	// For energy mapping
	for(size_t i=0;i<32;i++)
	{
		if(madc[1][i]>8096) madc[1][i]=0;
		if( adc[1][i]>8096) adc[1][i]=0;
		r2t2->y[i]=madc[1][i]*r2t2_ypar[i][1]+r2t2_ypar[i][0];
		r2t2->x[i]=adc[1][i]*r2t2_xpar[i][1]+r2t2_xpar[i][0];

		if(madc[0][i]>8096) madc[0][i]=0;
		if(madc[4][i]>8096) madc[4][i]=0;
		r2t1->y[i]=madc[0][i]*r2t1_ypar[i][1]+r2t1_ypar[i][0];//rnd3->Uniform(1);
		r2t1->x[i]=madc[4][i]*r2t1_xpar[i][1]+r2t1_xpar[i][0];
		
		if(madc[2][i]>8096) madc[2][i]=0;
		if(madc[5][i]>8096) madc[5][i]=0;
		l2t1->y[i]=madc[2][i]*l2t1_ypar[i][1]+l2t1_ypar[i][0];
		l2t1->x[i]=madc[5][i]*l2t1_xpar[i][1]+l2t1_xpar[i][0];

		if(madc[3][i]>8096) madc[3][i]=0;
		if( adc[2][i]>8096) adc[2][i]=0;
		l2t2->y[i]=madc[3][i]*l2t2_ypar[i][1]+l2t2_ypar[i][0];
		l2t2->x[i]=adc[2][i]*l2t2_xpar[i][1]+l2t2_xpar[i][0];
	}
	r2t2->HitEP(140.,47.,0.);
	r2t1->HitEP(226.,22.5,0.);
	l2t1->HitEP(226.,22.5,180.);
	l2t2->HitEP(140.,47.,180.);

	Int_t ref0=gdc[0][127][0];
	Int_t ref1=gdc[1][127][0];
	//??????????? r2t1->yt[0] ?????
	for(Int_t i=0;i<32;i++)
	{
		if(gmulti[0][i]==1) r2t1->yt[i]=-0.1*(ref0-(gdc[0][i][0]));
		if(gmulti[0][i+32]==1) r2t2->yt[i]=-0.1*(ref0-(gdc[0][i+32][0]));
		
		if(gmulti[1][i]==1) l2t1->yt[i]=-0.1*(ref1-(gdc[1][i][0]));
		if(gmulti[1][i+32]==1) l2t2->yt[i]=-0.1*(ref1-(gdc[1][i+32][0]));
	}
}

void Tele::MappingSSSD()
{
	// For energy mapping
	for(size_t i=0;i<16;i++)
	{
		if(madc[7][i+16]>8096) madc[7][i+16]=0;
		if(madc[7][i]>8096) madc[7][i]=0;
		if(madc[6][i]>8096) madc[6][i]=0;
		if(madc[6][i+16]>8096) madc[6][i+16]=0;
		r1t2->x[i]=madc[7][i+16]*r1t2_xpar[i][1]+r1t2_xpar[i][0];
		r1t1->x[i]=madc[7][i]*r1t1_xpar[i][1]+r1t1_xpar[i][0];//rnd3->Uniform(1);		
		l1t1->x[i]=madc[6][i]*l1t1_xpar[i][1]+l1t1_xpar[i][0];
		l1t2->x[i]=madc[6][i+16]*l1t2_xpar[i][1]+l1t2_xpar[i][0];
	}
	r1t2->HitEP(125.,47.,0.);
	r1t1->HitEP(211.,22.5,0.);
	l1t1->HitEP(211.,22.5,180.);
	l1t2->HitEP(125.,47.,180.);

	Int_t ref0=gdc[0][127][0];
	Int_t ref1=gdc[1][127][0];
	for(Int_t i=0;i<16;i++)
	{
		if(gmulti[0][96+i]>0) r1t1->xt[i]=-0.1*(ref0-(gdc[0][96+i][0]));
		if(gmulti[0][64+i]>0) r1t2->xt[i]=-0.1*(ref0-(gdc[0][64+i][0]));
		
		if(gmulti[1][96+i]>0) l1t1->xt[i]=-0.1*(ref1-(gdc[1][96+i][0]));
	}

}

void Tele::MappingSSD()
{
	// For energy mapping
	if(adc[0][0]>8096) adc[0][0]=0;
	if(adc[0][1]>8096) adc[0][1]=0;
	if(adc[0][2]>8096) adc[0][2]=0;
	if(adc[0][3]>8096) adc[0][3]=0;
	r3t2->xe=adc[0][0]*r3t2_par[0][1]+r3t2_par[0][0];
	r3t1->xe=adc[0][1]*r3t1_par[0][1]+r3t1_par[0][0];//rnd3->Uniform(1);		
	l3t1->xe=adc[0][2]*l3t1_par[0][1]+l3t1_par[0][0];
	l3t2->xe=adc[0][3]*l3t2_par[0][1]+l3t2_par[0][0];
}		

void Tele::ReadPar(const char *filename,Double_t par[][2],Int_t n)
{
	ifstream in(filename);
	for(int i=0;i<n;i++)
	{
		par[i][0]=0;
		par[i][1]=1;
	}
	if(!in.is_open()) 
	{
		cout<<"cannot open the file: "<<filename<<endl;
		return;
	}
	Int_t cnt(0);
	while(1)
	{
		Double_t tmpx,tmpy;
		in>>tmpx>>tmpy;
		if(!in.good()) break;
		par[cnt][0]=tmpx;
		par[cnt][1]=tmpy;
		cnt++;
	}
}




